﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Reflection;
using Telerik.OpenAccess.Metadata;
using Telerik.OpenAccess.Metadata.Fluent;

namespace OpenAccessFluent
{
    public static class FluentExtensions
    {

        private static string GetDBType(LambdaExpression expression, Backend backend)
        {
            string sqlType = string.Empty;

            var memberExpression = expression.Body as MemberExpression;

            if (memberExpression != null && memberExpression.Member != null)
            {
                MemberInfo info = (memberExpression.Member as MemberInfo);
                var attributes = info.GetCustomAttributes(typeof(BackendDBTypeAttribute), false);
                if (attributes.Count() == 1)
                {
                    var attribute = (attributes[0] as BackendDBTypeAttribute);
                    if (backend == Backend.MsSql)
                    {
                        sqlType = attribute.SQLDBType;
                    }
                    else if (backend == Backend.Oracle)
                    {
                        sqlType = attribute.OracleDBType;
                    }
                }
            }
            else
            {
                throw new ArgumentException("expression was not in the correct format");
            }
            if (string.IsNullOrEmpty(sqlType))
            {
                throw new InvalidProgramException("sql type could not be resolved");
            }

            return sqlType;
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, bool>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, bool?>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, short>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, short?>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, int>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, int?>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, long>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, long?>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, float>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, float?>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, double>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, double?>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, Guid>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static PrimitivePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, Guid?>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static DateTimePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, DateTime>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static DateTimePropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, DateTime?>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

        public static StringPropertyConfiguration HasProperty<TEntity>(this MappingConfiguration<TEntity> configuration, Expression<Func<TEntity, string>> expression, Backend backend)
        {
            string sqlType = GetDBType(expression, backend);
            return configuration.HasProperty(expression).HasColumnType(sqlType);
        }

    }
}
